; arena routines

!ifdef INCLUDE_ARENA !eof
!if INCLUDE_GUARD > 0 {
INCLUDE_ARENA = 1
}

!zn Arena {

AFLAG_MASK_OWNER      = %00000111   ; the owner (or player id for wizards)
AFLAG_MASK_ILLUSION   = %00010000   ; is illusionary (i.e. bit 4 1 = illusion, bit 4 0 = real)
AFLAG_MASK_DISBELIEVE = %00100000   ; has had disbelieve cast on it
AFLAG_MASK_UNDEAD     = %01000000
AFLAG_MASK_MOVED      = %10000000   ; "has moved" for creatures, "has been spread" for gooey blob and fire

DEAD_MASK_OWNER       = %00000111   ; the former owner of the creature
DEAD_MASK_IS_DEAD     = %00001000   ; creature is dead

arena_draw:
    jsr print_setup_dst_area
    ldy #0
arena_draw_loop:
    jsr arena_draw_square
    iny
    cpy #AREA_SIZE
    bne arena_draw_loop
    rts

; y = square
arena_draw_square:
    lda arena_aflag,y
    and #AFLAG_MASK_OWNER
    tax
    lda wizard_color,x
    sta color
    lda arena_above,y
    bne +
    sta color
    beq ++  ; bra
+   tax
    lda arena_dead,y
    and #DEAD_MASK_IS_DEAD
    beq +
    lda arena_dead,y
    and #DEAD_MASK_OWNER
    tax
    lda wizard_color,x
    sta color
    lda #CHAR_DEAD
    bne ++  ; bra
+   lda spellgfx,x
++  ldx tbl_area_i_to_screen,y
    jsr print_char_dst_x
    rts


; view arena, use cursor to select a square
; in:
; a = start coord ($ff for default)
; out:
; a = coord
; C = 1 if debug key pressed
arena_view:
    cmp #$ff
    bne +
    lda #AREA_W * (AREA_H / 2) + (AREA_W / 2)
+   sta acoord
    sta acoordprev
arena_view_keep_coord:

.arena_view_loop:
    ; erase old cursor
    jsr arena_erase_cursor

    ; limit coord
    lda acoord
    cmp #-AREA_W - 2
    bcc +
    clc
    adc #AREA_SIZE
+   cmp #AREA_SIZE
    bcc +
    sec
    sbc #AREA_SIZE
+   sta acoord
    sta acoordprev

    ; draw cursor
    tay
    ldx tbl_area_i_to_screen,y
    lda colorram + POS_AREA,x
    and #$f
    bne +
    ; blank, use wizard color and (reversed) space
    ldy player_i
    lda wizard_color,y
    sta colorram + POS_AREA,x
    lda #' ' | %10000000
    bne ++  ; bra
+   lda vmatrix + POS_AREA,x
    ora #%10000000 ; normal -> reversed
++  sta vmatrix + POS_AREA,x

    ; draw info on monster/wizard
    jsr examine_square

    ; phase callback
    bit flag_phase
    bpl +   ; not movement phase
    jsr movement_phase_arena_view_cb
    bcc .arena_view_input   ; bra
+   bvc +   ; not cast phase
    jsr cast_phase_arena_view_cb
    bcc .arena_view_input   ; bra
+
    ; get input
.arena_view_input:
!if DEBUG_ARENA_VIEW = 1 {
    ; show coord, x, y, distance
    lda acoord
    ldy #0
    jsr print_hex
    lda acoord
    jsr get_yx
    ldy #3
    jsr print_hex
    lda acoord
    jsr get_yx
    tya
    ldy #6
    jsr print_hex
    lda wcoord
    ldx acoord
    jsr get_distance
    ldy #9
    jsr print_hex
}
!if DEBUG_ARENA_VIEW = 2 {
    ; show coord, above, aflag, below, bflag, dead
    lda acoord
    ldy #0
    jsr print_hex
    ldx acoord
    lda arena_above,x
    ldy #3
    jsr print_hex
    lda arena_aflag,x
    ldy #6
    jsr print_hex
    lda arena_below,x
    ldy #9
    jsr print_hex
    lda arena_bflag,x
    ldy #12
    jsr print_hex
    lda arena_dead,x
    and #$f
    ldy #15
    jsr print_hex1
}
    jsr get_input_wait_irq
    beq .arena_view_input

    cmp #INPUT_SELECT
    bne ++
    jsr print_clear_info
    lda acoord
!if DEBUG_KEY > 0 {
    clc
}
    rts

!if DEBUG_KEY > 0 {
++  cmp #INPUT_DEBUG
    bne ++
    jsr print_clear_info
    lda acoord
    sec
    rts
}

++  ; all other nonzero INPUT_* are directions, add cursor offset from table
    tax
    lda acoord
    clc
    adc tbl_input_dir,x
    sta acoord
    jmp .arena_view_loop


arena_erase_cursor:
    ; erase old cursor
    ldy acoordprev
    ldx tbl_area_i_to_screen,y
    lda vmatrix + POS_AREA,x
    and #%01111111  ; reversed -> normal
    sta vmatrix + POS_AREA,x
    cmp #' '
    bne +
    lda #COLOR_BLACK
    sta colorram + POS_AREA,x
+   rts


; sets the index of the current player for spell casting
; in:
; player_i = current player
; out:
; y = wcoord = wizard coords
set_current_player_index:
    lda player_i
    clc
    adc #WIZARD_INDEX
    ldy #0
-   cmp arena_above,y
    beq +
    cmp arena_below,y
    beq +
    iny
    cpy #AREA_SIZE
    bne -
+   sty wcoord
    sty acoord
    sty scoord
    rts


; calculate the distance between 2 squares in the arena in "chaos range" terms (roughly 2*distance)
; calculate 2*(larger of xposdiff and yposdiff) + (the smaller of the 2)
; in:
; a = square 1
; x = square 2
; out:
; a = distance
get_distance:
    sty .get_distance_smc_y
    jsr setup_yxd
.get_distance_smc_y = * + 1
    ldy #0
    cmp temp_dx
    bcs +
    ; dy < dx  -> dx * 2 + dy
    asl temp_dx
    bcc ++  ; bra
+   ; dy >= dx -> dy * 2 + dx
    asl
++  ; clc
    adc temp_dx
    rts


; in:
; a = square 1
; x = square 2
; out:
; temp_[xy][12], temp_d[xy]
; a = temp_dy
; y = temp_y2
setup_yxd:
    jsr get_yx
    sta temp_x1
    sty temp_y1
    txa
    jsr get_yx
    sta temp_x2
    sty temp_y2
    ; abs(x2 - x1)
    ; sec
    sbc temp_x1
    bcs +
    eor #$ff
    ; clc
    adc #1
+   sta temp_dx
    ; abs(y2 - y1)
    tya
    sec
    sbc temp_y1
    bcs +
    eor #$ff
    ; clc
    adc #1
+   sta temp_dy
    rts


; in:
; a = coord
; out:
; a = x coord
; y = y coord
; C = 1
get_yx:
!if FAST_GETYX > 0 {
    tay
    lda tbl_area_i_y,y
    sta .get_yx_y
    lda tbl_area_i_x,y
.get_yx_y = * + 1
    ldy #0
    sec
} else {
    ldy #$ff
-   iny
    sec
    sbc #AREA_W
    bcs -
    ; clc
    adc #AREA_W
}
    rts


; called at the end of the moves round (fixme?) and on turmoil cast
; resets the movement flags so that the creatures can move next time
unset_moved_flags:
    ldy #AREA_SIZE
-   lda arena_aflag - 1,y
    and #<!AFLAG_MASK_MOVED
    sta arena_aflag - 1,y
    dey
    bne -
    rts


.surround_table_x:
!by  0,  1,  1,  1,  0, -1, -1, -1
.surround_table_y:
!by -1, -1,  0,  1,  1,  1,  0, -1

.y_to_pos:
!for .y, 0, AREA_H - 1 {
!by .y * AREA_W
}

; in:
; a = square
; x = direction (0..7)
; out:
; a = new square | $ff for invalid
apply_position_modifier:
    jsr get_yx
    clc
    adc .surround_table_x,x
    cmp #AREA_W
    bcc +
-   lda #$ff
    rts
+   sta .apply_position_modifier_x
    tya
    clc
    adc .surround_table_y,x
    cmp #AREA_H
    bcs -
    tay
    lda .y_to_pos,y
    clc
.apply_position_modifier_x = * + 1
    adc #0
    rts


; in:
; a = wizard spell ID (WIZARD_INDEX + 0..7)
destroy_all_creatures_id:
    sec
    sbc #WIZARD_INDEX
; in:
; a = wizard index (0..7)
destroy_all_creatures:
    sta enemy_wizard
    tax
    lda wizard_color,x
    sta color
    lda #0
    sta flag_anim   ; sound played
    lda #7
    sta zploops
.destroy_all_creatures_frame_loop:
    dec zploops
    bpl +
    jsr arena_draw
    rts
+   lda #5
    jsr delay
    ldy #-1
.destroy_all_creatures_square_loop:
    iny
    cpy #AREA_SIZE
    beq .destroy_all_creatures_frame_loop
    lda arena_above,y
    beq .destroy_all_creatures_square_loop
    cmp #WIZARD_INDEX
    bcs .destroy_all_creatures_square_loop
    lda arena_dead,y
    and #DEAD_MASK_IS_DEAD
    bne .destroy_all_creatures_square_loop
    ; check the owner
    lda arena_aflag,y
    and #AFLAG_MASK_OWNER
    cmp enemy_wizard
    beq +++
    ; check if it is a blob
    lda arena_above,y
    cmp #SPELL_GOOEY_BLOB
    bne .destroy_all_creatures_square_loop
    ; blob... is anything under it?
    lda arena_below,y
    beq .destroy_all_creatures_square_loop
    ; is the trapped creature the effected player?
    lda arena_bflag,y
    and #AFLAG_MASK_OWNER
    cmp enemy_wizard
    bne .destroy_all_creatures_square_loop
+++
    ; draw the pop frame at this creatures location
    ldx zploops
    lda gfx_anim_pop,x
    ldx tbl_area_i_to_screen,y
    sta vmatrix + POS_AREA,x
    lda color
    sta colorram + POS_AREA,x

    ; do a sound effect, once only
    bit flag_anim   ; sound played
    bmi +
    lda #SND_SPELLSUCCESS
    jsr sound_play
    dec flag_anim   ; sound played
+
    ; check if we are on the last loop
    lda zploops
    bne .destroy_all_creatures_square_loop

    ; last loop, get rid of the creature
    lda arena_aflag,y
    and #AFLAG_MASK_OWNER
    cmp enemy_wizard
    beq +
    lda #0
    sta arena_below,y
    sta arena_bflag,y
    beq .destroy_all_creatures_square_loop  ; bra
+   ; same owner as effected wiz
    lda arena_above,y
    cmp #SPELL_GOOEY_BLOB
    bne +
    lda arena_below,y
    beq +
    sta arena_above,y
    lda arena_bflag,y
    sta arena_aflag,y
    lda #0
    sta arena_below,y
    sta arena_bflag,y
    jmp .destroy_all_creatures_square_loop
+   lda arena_below,y
    pha
    lda #0
    sta arena_below,y
    pla
    beq +
    sta arena_above,y
    jmp .destroy_all_creatures_square_loop
+   lda arena_bflag,y
    sta arena_above,y
    beq +
    lda arena_dead,y
    ora #DEAD_MASK_IS_DEAD
    sta arena_dead,y
    lda #0
    sta arena_bflag,y
+   jmp .destroy_all_creatures_square_loop


; randomly destroy castles at the end of each casting round
destroy_castles:
    jsr print_setup_dst_area
    ldy #-1
--  iny
    cpy #AREA_SIZE
    bne +
    lda #20
    jsr delay
    rts
+   lda arena_above,y
    cmp #SPELL_MAGIC_CASTLE
    bcc --
    cmp #SPELL_DARK_CITADEL + 1
    bcs --
    jsr get_random_10
    cmp #8
    bcc --
    lda arena_below,y
    sta arena_above,y
    lda #0
    sta arena_below,y
    sty acoord
    ldx #COLOR_PURPLE
    jsr pop_animation
    lda #4
    jsr delay
    ldy acoord
    jmp --

.text_new_spell:
!tx "New spell for", 0

; give a player a spell from the magic wood
random_new_spell:
    ldy #-1
--  iny
    cpy #AREA_SIZE
    bne +
    rts
+   lda arena_above,y
    cmp #SPELL_MAGIC_WOOD
    bne --
    lda arena_below,y
    beq --
    sec
    sbc #WIZARD_INDEX
    sta player_i
    ; is a wood with someone inside
    jsr get_random_10
    cmp #7
    bcc --
    ; got here? then the wood has given us a spell!
    sty acoord
    jsr print_setup_dst_statusline
    jsr print_clear_status
    lda #<.text_new_spell
    sta zptxt
    lda #>.text_new_spell
    sta zptxth
    lda #COLOR_YELLOW
    sta color
    jsr print_packed_dst_ready
    ldx #COLS
    ldy player_i
    jsr print_wizard_name_offs_x
    lda #100
    jsr delay
    jsr print_clear_status
    jsr print_setup_dst_area
    ; generate new spell
    lda #SPELL_TURMOIL + 1 - SPELL_KING_COBRA  ; number of spells - first spell
    jsr get_random_a
    clc
    adc #SPELL_KING_COBRA   ; + first spell
    sta current_spell   ; new spell
    ; give the spell to the wizard
    ldy player_i
    ldx wizard_offs,y
    ldy acoord
    lda player_data + PDATA_OFFS_SPELL_COUNT,x
    cmp #NUM_SPELLS_MAX
    bcs +   ; no room for the spell
    stx zptmp
    ; clc
    adc zptmp
    inc player_data + PDATA_OFFS_SPELL_COUNT,x
    tax
    sta player_data + PDATA_OFFS_SPELLS,x
+
    lda arena_below,y
    sta arena_above,y
    lda player_i
    sta arena_aflag,y
    jsr arena_draw_square
    ldy acoord
    lda #20
    jsr delay
    jmp --

} ; !zn Arena
