; computer AI routines

!ifdef INCLUDE_COMPUTER !eof
!if INCLUDE_GUARD > 0 {
INCLUDE_COMPUTER = 1
}

!zn Computer {

attack_pref = * - 2 ; make table zero based
!by $08, $0b, $07, $09  ; 2 - King Cobra .. 5 - Crocodile
!by $09, $09, $0f, $0e
!by $06, $0e, $0b, $0a
!by $0d, $09, $10, $0b
!by $0f, $0f, $0f, $10
!by $15, $0f, $19, $19
!by $1b, $11, $12, $13
!by $0d, $09, $0b, $07
!by $04, $04, $04, $00
!by $01, $00, $00, $00
!by $09, $09, $09, $09, $09, $09, $09, $09    ; 42 - Wizards


reset_priority_table:
    ldy #AREA_SIZE
-   lda #$00
    sta tbl_prio - 1,y
    lda #$ff
    sta tbl_target - 1,y
    dey
    bne -
    rts


; fill the priority table with all squares in range from start index
; distance is from target index
; in:
; a = target index
; x = range
; scoord = start index
; out:
; a = number of squares
create_range_table:
    sta .create_range_table_target
    inx
    stx .create_range_table_range
    jsr reset_priority_table
    ldx #0  ; arena coord
    stx zptmp   ; table index
    stx zpdst   ; count
-   lda scoord
    jsr get_distance
.create_range_table_range = * + 1
    cmp #0
    bcs +
.create_range_table_target = * + 1
    lda #0
    jsr get_distance
    ldy zptmp   ; table index
    sta tbl_prio,y
    txa
    sta tbl_target,y
    iny
    sty zptmp   ; table index
    inc zpdst   ; count
+   inx
    cpx #AREA_SIZE
    bne -
    lda zpdst   ; count
    jsr order_table_target
    lda zpdst   ; count
    rts


; get the furthest square away still in range
; in:
; lut_index = end of table
; out:
; target_square_found = $80 if found
; acoord = target index
get_furthest_inrange:
    lda #0
    sta target_square_found
-   ldx lut_index
    dex
    stx lut_index
    cpx #$ff
    bne +
    rts
+   lda tbl_target,x
    sta acoord
    lda wcoord
    ldx #0
    jsr los_blocked
    bcc -
    ldx acoord
    lda arena_above,x
    beq +
    lda arena_dead,x
    and #DEAD_MASK_IS_DEAD
    beq -
+   ; nothing here or it is dead
    lda #$80
    sta target_square_found
    rts


; get the best index of the prio table
; get the furthest square away still in range
; in:
; lut_index = end of table
; out:
; C = 1 if target found (was: 0x53 ; 0x4b <-> C = 0)
; acoord = target
get_best_index:
    ldx lut_index
-   cpx #$ff
    bne +
    stx lut_index
    clc
    rts
+   lda tbl_target,x
    dex
    cmp #$ff
    bne +
    jmp -
+   sta acoord
    ; check xpos < AREA_W (XXX why?)
    jsr get_yx
    cmp #AREA_W
    bcs -
    stx lut_index
    sec
    rts


; create a table of the strongest wizards, based on quantity/quality of creatures
; out:
; a = index of target with largest priority
; tbl_target = targets sorted by priority
create_table_wizards:
    jsr reset_priority_table
    ldx #0
    ldy #0
-   lda arena_above,y
    sec
    sbc #WIZARD_INDEX
    bcc +
    jsr create_enemy_table_entry
+   lda arena_below,y
    sec
    sbc #WIZARD_INDEX
    bcc +
    jsr create_enemy_table_entry
+   iny
    cpy #AREA_SIZE
    bne -
    lda #7  ; txa should also work
    jsr order_table_target
    lda tbl_target
    rts

; in:
; a = wizard id (0..7)
; y = target index
; x = table index
; out:
; y = target index
; x = (next) table index
create_enemy_table_entry:
    cmp player_i
    bne +
    rts
+   sta .create_enemy_table_entry_wizid
    tya
    sta tbl_target,x
    ldy #0
-   lda arena_above,y
    beq ++
    cmp #SPELL_SHADOW_WOOD
    bcs ++
    lda arena_aflag,y
    and #AFLAG_MASK_OWNER
.create_enemy_table_entry_wizid = * + 1
    cmp #0
    beq ++
    lda arena_above,y
    stx zptmp
    tax
    lda attack_pref,x
    ldx zptmp
    lsr
    lsr
    clc
    adc priority_offset
    clc
    adc tbl_prio,x
    sta tbl_prio,x
++  iny
    cpy #AREA_SIZE
    bne -
    lda tbl_target,x
    tay
    inx
    rts


; check the square in index to see if it contains a live enemy creature
; in:
; y = coord
; scoord = wizard (or creature?) coord
; out:
; a = "creature attack preference"
; zptmp = range (if Z == 0)
contains_enemy:
    lda arena_above,y
    cmp #SPELL_KING_COBRA
    bcs +
-   lda #0
    rts
+   stx .contains_enemy_smc_x
    tax ; creature
    lda arena_dead,y
    and #DEAD_MASK_IS_DEAD
    bne ++
    lda arena_aflag,y
    and #AFLAG_MASK_OWNER
    cmp player_i
    beq ++
    ; got to here? contains a living enemy creature
    ; get the "creature attack preference" value
    lda attack_pref,x
    clc
    adc priority_offset
    pha ; pref
    tya
    ldx scoord
    jsr get_distance
    sta zptmp   ; range
    pla ; pref
    sec
    sbc zptmp   ; range
    bcs +++
++
    lda #0
+++
.contains_enemy_smc_x = * + 1
    ldx #0
    rts

; in:
; scoord = wizard (or creature?) coord
; out:
; a = index of target with largest priority
; tbl_target = targets sorted by priority
; x = number of enemies
create_table_enemies:
    jsr reset_priority_table
    ldx #0
    ldy #0
-   jsr contains_enemy
    cmp #0
    beq ++
    lsr zptmp   ; range>>=1
    clc
    adc #$14    ; ? max range/2 ?
    sec
    sbc zptmp
    sta tbl_prio,x
    tya
    sta tbl_target,x
    inx
++  iny
    cpy #AREA_SIZE
    bne -
    lda tbl_target
    rts


; in:
; a = attacker index (or $ff if no attacker ready)
; out:
; a = acoord = index of target
get_strongest_wizard:
    sta attacker_index
    ldx player_offs
    lda player_data + PDATA_OFFS_TIMID,x
    cmp #10 ; ?
    bcc ++
    ; create the priority table based on "strongest wizard" - one with most/best creatures
-   jsr create_table_wizards
    sta acoord
    rts
++  ; we have less than 10 in the timid value,
    ; go for the wizard who's creature is closest to us
    jsr create_table_enemies
    ; the best value will now be the index of the creature closest to us
    ; or the wizard closest to us, as they are treated the same in the create_table_enemies routine
    lda tbl_prio
    beq -   ; no dangerous creature found
    ; first of all, get the most dangerous creature... regardless of whether we can kill it or not
    ; if we can't kill it, later we will attack its owner to do away with it
    txa ; count
    jsr order_table_target
    ldy tbl_target
    sty enemy_creature
    ; find owner
    lda arena_aflag,y
    and #AFLAG_MASK_OWNER
    ;sta enemy_wizard
    clc
    adc #WIZARD_INDEX
    ldx #0
-   cmp arena_above,x
    beq +
    cmp arena_below,x
    beq +
    inx
    bne -   ; bra
+   stx enemy_wizard
    ; (chaos-0.82 calculated enemy_wizard and enemy_creature range here, but the latter is changed below!)
    ; Magic Fire close to a wizard confuses the enemy creatures
    ; Try this out, so that fire is not attractive
    ; Also, undead creatures should not be attractive to living ones
    lda #0
    sta flag_anim   ; attacker_undead
    ldx attacker_index
    cpx #$ff
    bne +
    ; a creature spell to cast... so use the spellid
    lda current_spell
    jsr is_undead_a
    jmp ++
+   ; is a valid square, see if the attacking square is undead
    jsr is_undead_x
++  ror flag_anim   ; attacker_undead, $80 | $00

    tya ; enemy creature
    tax ; enemy creature
    ldy #0  ; prio index
-   cpx #$ff
    beq +++
    lda tbl_target,y
    tax ; enemy creature
    lda arena_above,x
    cmp #SPELL_MAGIC_FIRE
    beq ++
    ; not magic fire so check undeadness
    ; remember that we are still scared of magic fire and undead creatures
    ; just that we are not worried about attacking it
    jsr is_undead_x
    bcc +++
    bit flag_anim   ; attacker_undead, $80 | $00
    bmi +++
++  iny
    bne -   ; bra
+++
    cpx #$ff
    bne +
-   lda enemy_wizard
    sta acoord
    rts
+   stx enemy_creature
    ; enemy wizard = index to the enemy wizard attacking us
    ; enemy creature = index to the enemy creature attacking us
    ; now see which is closer and attack them
    lda scoord
    ldx enemy_wizard
    jsr get_distance
    sta zptmp   ; wizard range
    lda scoord
    ldx enemy_creature
    jsr get_distance
    cmp zptmp   ; wizard range
    bcs -   ; creature range >= wizard range
    lda enemy_creature
    sta acoord
    rts

; in:
; x = creature coord
is_undead_x:
    lda arena_aflag,x
    and #AFLAG_MASK_UNDEAD
    bne +
    lda arena_above,x
    ; FALL THROUGH

; in:
; a = creature
; out:
; C = 1 if undead
is_undead_a:
    cmp #SPELL_VAMPIRE
    bcc +++
    cmp #SPELL_ZOMBIE + 1
    bcs ++
+   sec
    rts
++  clc
+++ rts


do_ai_spell:
    ;    set spell 0 priorty = 12 + GetRand(10)
    ;    order the spell list by priority
    ;    best spell = 0
    ;    while (best spell < spell list length) {
    ;      select the best spell
    ;      select the best square for this spell
    ;      if no square is good
    ;        best spell ++
    ;      else
    ;        break
    ;    }
    ;    if a spell has been chosen...
    ;      cast spell at chosen square
    ;      remove casted spell from list

    jsr update_creaturecount

    jsr get_wizard_spells
    sta num_spells

    ; get spell priorities
    tay
    dey
-   lda (optr),y
    asl
    tax
    lda spelldata_packed,x
    and #$f
    tax
    lda tbl_spelldata_aiprio,x
    sta tbl_prio,y
    dey
    bne -
    sty best_spell

    ; spell 0 priority is random
    ; (on first turn this is disbelieve)
    jsr get_random_10
    clc
    adc #12
    sta tbl_prio
    ; order spells, max priority will be on spell slot 0
    lda num_spells
    ldx optr
    ldy optrh
    jsr order_table
--  ldy best_spell
    cpy num_spells
    bne +
    ; no (suitable) spells
    rts
+
    lda #0
    sta temp_cast_amount

    lda (optr),y
!if DEBUG_AI_SPELL > 0 {
    bne +   ; should always jump
-
!by $02 ; JAM
+   cmp #SPELL_TURMOIL + 1
    bcs -   ; should never jump
}
    jsr set_spell

    ; "cast" the spell... each casting routine has the CPU AI code for that spell built into it.
    lda #$80
    sta target_square_found
    jsr set_current_spell_chance
    lda current_spell
    jsr spell_cast

    ; if no good square was found, then go to the next spell
    bit target_square_found
    bmi +
    inc best_spell
    bne --  ; bra
+   ; spell was cast succesfully, remove it (if not disbelieve)
    lda best_spell
    clc
    adc player_offs
    tay
    lda current_spell
    ldx player_offs
    jsr remove_spell
    rts


; the computer tries to cast a creature
ai_cast_creature:
    ; get the square closest to the best target
    lda #$ff
    jsr get_strongest_wizard

    lda current_spell
    jsr spell_get_range
    tax
    lda acoord
    jsr create_range_table
    sta lut_index
    jsr get_furthest_inrange

    bit target_square_found
    bmi +
ai_cast_done:
    lda #0
    sta temp_cast_amount
    rts
+   jsr .ai_cast_print_name_delay
    jsr spell_animation
    lda #0
    sta temp_illusion_flag

    lda current_spell
    cmp #SPELL_GOOEY_BLOB
    bcs +++
    ; randomly cast illusions, but more chance of trying if the spell is tricky to cast
    lda current_spell_chance
    cmp #4
    bcs +
    jsr get_random_10
    cmp #6
    bcs ++
+   jsr get_random_10
    cmp #8
    bcc +++
++  dec temp_illusion_flag
+++

    jsr set_spell_success
    bit temp_success_flag
    bpl +
    ldx acoord
    jsr creature_spell_succeeds
    ldy acoord
    jsr arena_draw_square
+   jsr cast_print_success_status
    lda #10
    jsr delay
    jmp ai_cast_done


ai_cast_wall:
    lda #4
    sta temp_cast_amount
    jsr create_table_enemies
    txa
    jsr order_table_target

    ldy #0  ; lut index
-   ldx tbl_target,y
    cpx #$ff
    bne +
    ; no decent square
    jmp ai_cast_nope
+   lda arena_above,x
    cmp #SPELL_SPECTRE
    bcs +
    cmp #SPELL_PEGASUS
    bcc +
    iny
    bne -   ; bra
+   stx acoord
    ; got to here? OK, a "good" square was found for wall.
    ; calculate spell success and update world chaos ...
    jsr set_spell_success
    jsr .ai_cast_print_name_delay
    ; TODO clear_message
    lda acoord
    ldx #9  ; ?? wall has range 13
    jsr create_range_table
    sta lut_index

    lda #0
    sta target_square_found
    sta tmp_square_found

    ; target_square_found is used as a temporary flag here
    ; tmp_square_found indicates if we find any castable-to squares for this spell at all
--- lda temp_cast_amount
    bne +
-   lda #0
    sta temp_cast_amount
    lda tmp_square_found
    sta target_square_found
    rts
+   jsr get_furthest_inrange
    bit target_square_found
    bpl -   ; we have run out of chackable squares!
    ; check that this is a valid square for walls
    ldx acoord
    lda arena_above,x
    bne ---
    jsr is_wall_adjacent
    bcs ---
    lda wcoord
    ldx #0  ; check LOS
    jsr los_blocked
    bcc ---
    ; we have found a suitable square
    lda #$80
    sta tmp_square_found
    jsr do_wall_cast
    jmp ---


; called at the start of lightning casting amongst other stuff...
; creates the priority table based on all enemy creatures and wizards
; wizards are rated higher than creatures and table is ordered
create_all_enemies_table:
    lda #$3c
    sta priority_offset
    ; create the priority table based on "strongest wizard"
    jsr create_table_wizards
    lda #$20
    sta priority_offset
    ldx #7
    ldy #0
-   jsr contains_enemy
    cmp #0
    beq +
    sta tbl_prio,x
    tya
    sta tbl_target,x
    inx
+   iny
    cpy #AREA_SIZE
    bne -
    txa ; size
    jsr order_table_target
    lda #0
    sta priority_offset
    sta lut_index
    rts

; in:
; x = target coord
; out:
; acoord = target coord
; C = 1 if range and LOS ok
ai_check_range_los:
    stx acoord
    jsr is_spell_in_range   ; in range?
    bcc +
    clc
    rts
+   lda wcoord
    ldx #0  ; check LOS
    jsr los_blocked
    rts

ai_cast_magic_missile:
    jsr create_all_enemies_table
    ldy #-1
-   iny
    ldx tbl_target,y
    cpx #$ff
    bne +
    jmp ai_cast_nope
+   lda arena_above,x
    cmp #SPELL_SHADOW_WOOD
    beq +
    cmp #WIZARD_INDEX
    bcs +
    cmp #SPELL_MAGIC_FIRE
    bcs -
+   jsr ai_check_range_los
    bcc -
    ; spell is good
    lda #0
    sta temp_cast_amount
    jsr .ai_cast_target_found_print_name_delay
    jmp do_magic_missile


ai_cast_subversion:
    lda #1
    sta temp_cast_amount
    jsr create_table_enemies
    txa
    jsr order_table_target
    ldy #-1
-   iny
    ldx tbl_target,y
    cpx #$ff
    bne +
    ; no decent square
    jmp ai_cast_nope
+   lda arena_above,x
    beq -
    cmp #SPELL_GOOEY_BLOB
    bcs -
    lda arena_below,x
    bne -
    lda arena_aflag,x
    and #AFLAG_MASK_DISBELIEVE
    bne -
    ; a valid creature for subversion...
    ; compare the start and end locations to see if in spell range
    jsr ai_check_range_los
    bcc -
    ; in range, have los and is not an illusion (because some one has cast disbelieve on it)
    lda #$80
    sta target_square_found ; FIXME already set by do_ai_spell?
    rts


ai_cast_disbelieve:
    jsr create_all_enemies_table
    ldy #-1
-   iny
    ldx tbl_target,y
    cpx #$ff
    bne +
    jmp ai_cast_nope
+   stx acoord
    lda arena_above,x
    cmp #SPELL_GOOEY_BLOB
    bcs -
    lda arena_aflag,x
    and #AFLAG_MASK_DISBELIEVE
    bne -
    ; is a creature and has not had disbeleive cast on it yet
.ai_cast_target_found_print_name_delay:
    lda #$80
    sta target_square_found ; FIXME already set by do_ai_spell?
.ai_cast_print_name_delay:
    jsr cast_print_spell_name
    lda #80
    jsr delay
    rts

ai_cast_justice:
    jsr .ai_cast_target_found_print_name_delay
    lda #$80
    sta target_square_found ; FIXME already set by do_ai_spell?
    jsr set_spell_success
    bit temp_success_flag
    bmi +
    jsr cast_print_success_status
    lda #0
    sta temp_cast_amount
    rts
+
.ai_cast_justice_cast_loop:
    lda temp_cast_amount
    beq .ai_cast_justice_done
    jsr create_all_enemies_table
    ldy #-1
.ai_cast_justice_lut_loop:
    iny
    ldx tbl_target,y
    cpx #$ff
    beq .ai_cast_justice_done
    ; check the creature at this index
    lda arena_above,x
    beq .ai_cast_justice_lut_loop
    cmp #WIZARD_INDEX
    bcs +
    cmp #SPELL_GOOEY_BLOB
    bcs .ai_cast_justice_lut_loop
+   ; wizard or proper creature
    stx acoord
    lda #60
    jsr delay
    jsr do_justice_cast
    dec temp_cast_amount
    bne .ai_cast_justice_cast_loop
.ai_cast_justice_done:
    rts


ai_cast_raisedead:
    lda #0
    sta target_square_found
    jsr reset_priority_table
    ldy #0  ; coord
    ldx #0  ; lut index
-   lda arena_dead,y
    and #DEAD_MASK_IS_DEAD
    beq +
    tya
    sta tbl_target,x
    stx zptmp
    ldx arena_above,y
    lda attack_pref,x
    ldx zptmp
    sta tbl_prio,x
    inx
+   iny
    cpy #AREA_SIZE
    bne -
    txa
    jsr order_table_target

    ; get the best value and try and cast it
    ; get the best index for each cast
    ldy #-1
--- iny
    ldx tbl_target,y
    cpx #$ff
    bne +
    rts
+
    ; check the creature at this index
    lda arena_above,x
    beq ---
    cmp #WIZARD_INDEX
    bcs +
    cmp #SPELL_GOOEY_BLOB
    bcs ---
+   ; wizard or proper creature
    jsr ai_check_range_los  ; stores x to acoord
    bcc ---
    ; got to here? good, print name and spell and do cast
    jsr .ai_cast_print_name_delay
    jsr do_raisedead_cast
    lda #0
    sta temp_cast_amount
    lda #$80
    sta target_square_found
    rts


ai_cast_turmoil:
    jsr create_table_enemies
    txa
    jsr order_table_target
    lda wcoord
    sta scoord
    ; this first part depends not on the index values, like most other spells.
    ; it instead depends on the priority value... the thinking behind this is:
    ; if there aren't enough enemy creatures really close, it isn't worth casting turmoil
    ldy #0
    sty .ai_cast_turmoil_danger
-   lda tbl_prio,y
    bne +
    ; not enough targets/danger
    lda #0
    sta target_square_found
    sta temp_cast_amount
    rts
+   clc
.ai_cast_turmoil_danger = * + 1
    adc #0
    sta .ai_cast_turmoil_danger
    cmp #$1e
    bcs +
    iny
    bne -   ; bra
+   ; the spell is worth casting
    jsr set_spell_success
    lda #1
    sta temp_cast_amount
    jmp .ai_cast_target_found_print_name_delay


ai_cast_nope:
    lda #0
    sta temp_cast_amount
    sta target_square_found
    rts


do_ai_movement:
    ;   if wiz has creatures to do his bidding...
    ;
    ;     get current position, set up relative
    ;     movement advantages for squares we can go to
    ;
    ;     if we are in a wood or castle, stay there
    ;     else ...
    ;     select the wizard
    ;     do the movement
    ;
    ;   end if
    ;
    ;   for all squares on the board...
    ;    if we have a creature here
    ;    then try selecting it
    ;      if we selected it successfully (it hadn't moved already)
    ;        then do the movement for this creature
    ;   end for
    ;
    ;   so if the wizard is creatureless, he himself acts like a normal creature

    jsr update_creaturecount

    ldx player_offs
    lda player_data + PDATA_OFFS_TIMID,x
    bne do_ai_movement_creatures
    jsr setup_wizard_move
    ldy scoord
    lda arena_above,y
    cmp #SPELL_MAGIC_WOOD
    beq +
    cmp #SPELL_MAGIC_CASTLE
    beq +
    cmp #SPELL_DARK_CITADEL
    bne ++
+   ; stay in wood/castle/citadel
    lda arena_aflag,y
    ora #AFLAG_MASK_MOVED
    sta arena_aflag,y
    bne do_ai_movement_creatures    ; bra
++  ; move wizard
    lda #0
    sta has_wizard_moved
    lda #$ff
    sta move_table_created  ; redundant?
    jsr movement_a_press    ; select the wizard
    lda #30
    jsr delay
    ; now try moving it to the target square
    jsr do_this_movement
    lda #10
    jsr delay

do_ai_movement_creatures:
    lda #$ff
    sta has_wizard_moved
    ; move creatures
    ldy #0
-   sty scoord
    sty acoord
    sty tmp_scoord
    jsr movement_a_press    ; select the creature
    lda selected_creature
    beq +
    lda #30
    jsr delay
    jsr do_this_movement
+   ldy tmp_scoord
    iny
    cpy #AREA_SIZE
    bne -
    lda #20
    jsr delay
    clc
    rts


do_this_movement:
    lda selected_creature
    bne +
    jmp do_this_movement_range_attack
+   bit tmp_is_flying
    bpl do_walking_move
    jsr do_flying_move
    jsr arena_draw  ; NOTE draws more than needed
    ; what if it becomes engaged?
    lda tmp_range_attack
    beq do_this_movement
    jmp do_this_movement_range_attack
do_walking_move:
    ; walking creature
    bit has_wizard_moved
    bmi ++
    ; wizard has not moved yet
    bit move_table_created
    bmi +++
    ; need to create the wizard movement table
    jsr setup_wizard_move
    jmp +++
++  ; wizard has moved already
    bit move_table_created
    bmi +++
    ; need to create the creature movement table
    jsr setup_creature_move
+++ ; get the first best square to move to
    ldy lut_index
    dey
    sty lut_index
    cpy #$ff
    bne ++
    inc lut_index
-   jsr movement_cancel
    ; if we have range attack after our movement, make sure it is handled properly
    lda tmp_range_attack
    bne do_this_movement_range_attack
    rts
++  ldx tbl_target,y
    lda scoord
    jsr apply_position_modifier
    sta acoord
    jsr movement_a_press
    jsr arena_draw  ; NOTE draws more than needed
    lda #4
    jsr delay
    lda tmp_range_attack
    bne do_this_movement_range_attack
    ; get the next square to move to
    ; note that movement_a_press sets "move_table_created" to 0,
    ; so after a move, we need to remake the table.
    ldy lut_index
    beq -   ; no good square to go to
    jmp do_this_movement
do_this_movement_range_attack:
--  lda tmp_range_attack
    bne +
    jsr arena_draw  ; NOTE draws more than needed
    lda #20
    jsr delay
    rts
+   ; range attack... this is identical for flying or not, wiz or not
    jsr get_best_rangeattack
    bcs +
    ; the best square is crap
    jmp movement_cancel
+   ; the target square is good
    jsr movement_a_press
    jsr arena_draw  ; NOTE draws more than needed
    jmp --


setup_creature_move:
    jsr reset_priority_table
    lda acoord  ; TODO check where this is set
    sta tmp_acoord
    jsr get_strongest_wizard
    jsr get_surrounding_square_prios
    lda tmp_acoord
    sta acoord
    sta scoord
    lda #$ff
    sta move_table_created
    rts

; get the priorities of squares surrounding "index" based on distance from "strongest"
; in:
; a = strongest index
; tmp_acoord = current index
get_surrounding_square_prios:
    sta .get_surrounding_square_prios_smc_strongest
    jsr reset_priority_table
    ldy #0
    sty lut_index
    ldx #7
--  lda tmp_acoord
    jsr apply_position_modifier
    cmp #$ff
    beq ++
    sta zptxt   ; target
    stx zptmp   ; pos mod i
.get_surrounding_square_prios_smc_strongest = * + 1
    ldx #0
    jsr get_distance
    ; add a check in here to make sure we don't
    ; tread on a magic wood. This table is ordered largest
    ; first. But the potential squares are read starting
    ; at the end (lut_index, then lut_index-1, etc)
    tax ; distance
    ldy zptxt   ; target
    lda arena_above,y
    cmp #SPELL_MAGIC_WOOD
    bne +
    inx ; "increase" the distance so it isn't as appealing ; careful not to increase too much, may go backwards!
+   txa ; distance
    ldy lut_index
    sta tbl_prio,y
    ldx zptmp   ; pos mod i
    txa
    sta tbl_target,y
    inc lut_index
++  dex
    bpl --
    lda lut_index
    jsr order_table_target
    rts


; get the best square for range attack
; in:
; lut_index
; out:
; C = 1 if found
; acoord = target if found
get_best_rangeattack:
    ; NOTE lut_index is reset in end_movement -> create_all_enemies_table
--  ldy lut_index
    lda tbl_target,y
    cmp #$ff
    bne +
    clc ; not found
    rts
+   tax ; target
    lda arena_above,x
    cmp #WIZARD_INDEX
    bcs ++
    cmp #SPELL_MAGIC_FIRE
    bcc ++
    cmp #SPELL_SHADOW_WOOD
    beq ++
    cmp #SPELL_MAGIC_WOOD
    bne +++
    lda arena_below,x
    beq +++
++  ; wizard, or wizard in wood, shadow wood or an actual creature
    ; anything >= magic fire will be ignored
    ; check if we are trying to attack an undead creature...
    ; original chaos didn't check the arena_aflag value for raise deaded creatures
    ; wizards can always attack undead with their magic bow range attacks
    jsr is_undead_x ; check defender
    bcc ++  ; jump if defender not undead
    lda selected_creature
    cmp #WIZARD_INDEX
    bcs ++
    ldy scoord
    lda arena_aflag,y
    and #AFLAG_MASK_UNDEAD
    beq +++
++  stx acoord
    inc lut_index
    sec ; found
    rts
+++ inc lut_index
    bne --  ; bra
    rts


do_flying_move:
    bit tmp_engaged_flag
    bpl +
    ; If wiz is flying and engaged to undead, causes problems
    lda #3
    sta tmp_movement_allowance
+   bit flying_target_found
    bmi do_flying_move_got_target
    bit has_wizard_moved
    bmi ++
    ; wizard has not moved yet
    bit move_table_created
    bmi +
    ; need to create the wizard movement table
    jsr setup_wizard_move
+   ; gets the "worst" square for the wizard (i.e. least dangerous)
    ldx lut_index
-   dex
    cpx #AREA_SIZE
    bcc +
    jmp movement_cancel
+   lda tbl_target,x
    cmp #$ff
    beq -
    ; strongest_index == a
    jmp +++
++  ; wizard has moved already
    ; need to create the creature movement table
    jsr reset_priority_table
    lda tmp_scoord
    jsr get_strongest_wizard
    ; strongest_index == a
+++ ldx tmp_scoord
    stx scoord
    ; for creatures, strongest_index (a)  will be the one closest to the "best" enemy
    ; for wizards, will be the safest square in range
    ldx tmp_movement_allowance
    jsr create_range_table
    sta lut_index
    lda #$ff
    sta flying_target_found

do_flying_move_got_target:
    jsr get_best_index
    bcs +
    jmp movement_cancel ; ran out
+   ; found one! acoord contains the target square
    ldx tmp_scoord
    stx scoord
    lda #$ff
    sta flying_target_found
    jsr movement_a_press
    lda #30
    jsr delay
    rts


; out:
; tbl_prio,target
; lut_index = number of targets
setup_wizard_move:
    jsr reset_priority_table
    ldy wcoord
    sty acoord
    sty scoord
    ; check the value in arena_above at the wizard's location
    lda arena_above,y
    cmp #SPELL_PEGASUS
    bcc ++
    cmp #SPELL_MANTICORE + 1
    bcs ++
    ; wizard on a flying mount
    clc ; SPECIAL_MOVE_MOUNT
    jmp wizard_flying_move
++  ; not on a flying mount (may be on a regular one though)
    cmp #WIZARD_INDEX
    bcc ++
    ldx player_offs
    lda player_data + PDATA_OFFS_FLAGS1,x
    and #PDATA_MASK_FLAGS1_WINGS
    beq ++
    ; If wiz is flying and engaged to undead, causes problems
    ; TODO bug - this is unset on timid == 0 path!
    ; NOTE select_creature sets it
    bit tmp_is_flying
    bpl ++
    ; has magic wings and is in the open
    sec ; SPECIAL_MOVE_WINGS
    jmp wizard_flying_move
++  lda #0
    sta lut_index
    ldx #7
--  lda acoord
    jsr apply_position_modifier
    stx .setup_wizard_move_smc_pos
    cmp #$ff
    beq +++
    ldx lut_index
    sta tbl_target,x
    tay
    lda arena_above,y
    cmp #SPELL_MAGIC_CASTLE
    bcc +
    cmp #SPELL_DARK_CITADEL + 1
    bcs +
    lda #$ff
    sta target_square_found
    lda #0  ; prio
    beq ++  ; bra
+   cmp #SPELL_MAGIC_WOOD
    bne +
    lda #$ff
    sta target_square_found
    lda #1  ; prio
    bne ++  ; bra
+   cmp #SPELL_HORSE
    bcc +
    cmp #SPELL_MANTICORE + 1
    bcs +
    lda arena_aflag,y
    and #AFLAG_MASK_OWNER
    cmp player_i
    bne +
    lda arena_below,y
    bne +
    ; is a mount and is ours and no one is on it
    lda #$ff
    sta target_square_found
    lda #2  ; prio
    bne ++  ; bra
+   ; is none of the above
    tya
    tax
    jsr get_priority_val
    clc
    adc #3
    ldx lut_index
++  sta tbl_prio,x
    inc lut_index
.setup_wizard_move_smc_pos = * + 1
+++ ldx #0
    dex
    bpl --
    lda lut_index
    jsr order_table_target
    lda wcoord
    sta acoord
    sta scoord
    lda #$ff
    sta move_table_created
    rts


; in:
; C = 0 if SPECIAL_MOVE_MOUNT, 1 if SPECIAL_MOVE_WINGS
; a = creature if C == 0
wizard_flying_move:
    ldx #$d ; wings
    bcs +
    ; mount
    asl
    asl
    tax
    lda creaturedata_packed + 2,x
    lsr
    lsr
    lsr
    lsr
    tax
    inx
+   stx tmp_movement_allowance
    inx
    stx .wizard_flying_move_smc_cmp ; tmp_movement_allowance + 1
    lda #$ff
    sta has_wizard_moved
    ; jsr reset_priority_table    ; redundant as this is always called via setup_wizard_move
    lda #0
    sta lut_index
    ; loop over arena to get priority of all squares in range
    ldx #0
--  ; get the distance from wizard start square to this square
    lda wcoord
    jsr get_distance
.wizard_flying_move_smc_cmp = * + 1
    cmp #0  ; tmp_movement_allowance + 1
    bcs +
    ; is in range of the wiz start square...
    ; find the distance/danger of all enemies relative to this square
    ; store as priority
    jsr get_priority_val
    ldy lut_index
    sta tbl_prio,y
    txa
    sta tbl_target,y
    inc lut_index
+   inx
    cpx #AREA_SIZE
    bne --
    lda lut_index
    jmp order_table_target


; gets the "danger" of the square at coord
; this is calculated based on enemy creature distance and how dangerous that creature is
; inout:
; x = coord
; out:
; a = priority
get_priority_val:
    stx scoord
    ldy #0
    sty .get_priority_val_smc_total
-   jsr contains_enemy
    clc
.get_priority_val_smc_total = * + 1
    adc #0
    sta .get_priority_val_smc_total
    iny
    cpy #AREA_SIZE
    bne -
    rts


; check if the cpu could dismount (traditionally, the cpu never dismounts)
; in:
; a = selected_creature = mount/castle we are on
; acoord = coord
; out:
; a = dismount yes/no $ff/0
cpu_dismount:
    ; which type of mount are we on?
    cmp #SPELL_PEGASUS
    bcc +
    cmp #SPELL_MANTICORE + 1
    bcc cpu_dismount_no ; too complex to work out
+   ldx #7
-   lda acoord
    jsr apply_position_modifier
    cmp #$ff
    beq ++
    ; rules for leaving our mount are consistent with the
    ; movement patterns when not mounted, i.e. prefer
    ; to move to a castle or tree before getting on a horse.
    tay
    lda arena_above,y
    cmp #SPELL_MAGIC_CASTLE
    bcc +
    cmp #SPELL_DARK_CITADEL + 1
    bcc cpu_dismount_yes
+   cmp #SPELL_MAGIC_WOOD
    beq cpu_dismount_yes
    cmp #SPELL_PEGASUS
    bcc ++
    cmp #SPELL_MANTICORE + 1
    bcs ++
    lda arena_aflag,y
    and #AFLAG_MASK_OWNER
    cmp player_i
    bne ++
    lda arena_below,y
    beq cpu_dismount_yes
++  dex
    bpl -
cpu_dismount_no:
    lda #0
    rts
cpu_dismount_yes:
    lda #$ff
    rts

} ; !zn Computer
