; movement phase

!ifdef INCLUDE_MOVEMENT !eof
!if INCLUDE_GUARD > 0 {
INCLUDE_MOVEMENT = 1
}

!zn Movement {

.text_turnmenu:
!tx TTOK_CONTINUE, 0
!tx "Next creature", 0
!tx "End turn", 0
!tx 0

.text_dismountmenu:
!tx TTOK_CONTINUE, 0
!tx "Dismount", 0
!tx 0

.text_cancelmenu:
!tx TTOK_CONTINUE, 0
!tx TTOK_CANCEL_, 0
!tx 0

.text_move:
!tx "Move", 0
.text_fly:
!tx "Fly", 0

.text_engaged:
!tx "Engaged in combat",0


movement_prepare:
    jsr print_clear_status
    jsr unset_moved_flags
    jsr spread_fire_blob
    jsr destroy_castles
    jsr random_new_spell
    rts

; out:
; a = 0
movement_clear_vars:
    lda #0
    sta selected_creature
    sta attacker
    sta tmp_range_attack
    sta tmp_engaged_flag
    sta tmp_creature_id
    sta tmp_range_attack_val
    sta tmp_movement_allowance
    sta flying_target_found
    sta tmp_is_flying
    sta tmp_wizard_movement_allowance
    rts

movement_phase:
    jsr movement_clear_vars
    jsr set_current_player_index

    jsr print_clear_status
    jsr print_setup_dst_statusline
    ldy player_i
    jsr print_wizard_name

    ; TODO highlight_players_stuff

    bit flag_player_cpu
    bpl +
    lda #SND_CPUSTART
    jsr sound_play
+
    lda #24
    jsr delay

    bit flag_player_cpu
    bpl +
    lda #10
    jsr delay
    jmp do_ai_movement
+
do_human_movement:
    jsr print_clear_status
    jsr print_setup_dst_statusline
    ldy player_i
    jsr print_wizard_name
    ; nothing selected
    jsr arena_view_keep_coord
    jsr select_creature
    bcs movement_is_selected
    ; not a valid selection
    ; show menu
    lda #COLOR_WHITE
    sta color
    jsr print_clear_status12
    ldx #<.text_turnmenu
    ldy #>.text_turnmenu
    jsr statusmenu
    cpy #1
    bcc do_human_movement
    bne movement_end_turn
    jsr movement_find_next_creature
    jmp do_human_movement
movement_end_turn:
    jsr arena_draw
    jsr print_clear_status
    clc
    rts
movement_is_selected:
    ; creature is selected for movement
    lda #SND_CHOSEN
    jsr sound_play
movement_is_selected_loop:
    jsr movement_print_status
    jsr arena_view_keep_coord
    ; show cancel menu if self selected
    cmp scoord
    bne +++
    lda #COLOR_WHITE
    sta color
    jsr print_clear_status12
    ldx #<.text_cancelmenu
    ldy #>.text_cancelmenu
    jsr statusmenu
    cpy #1
    bcc movement_is_selected_loop
    jsr movement_cancel
    beq do_human_movement
    bne movement_is_selected_loop
+++ ; move/attack
    jsr movement_a_press_selected
movement_is_selected_check:
    jsr arena_draw
    ; move until selected_creature == 0 (|| tmp_range_attack == 0)
    lda selected_creature
    bne movement_is_selected_loop
    lda tmp_range_attack    ; TODO redundant?
    bne movement_is_selected_loop
    jmp do_human_movement


movement_a_press:
    lda acoord
    cmp #$ff
    bne +
    rts
+   lda selected_creature
    bne ++
    jsr select_creature
    bcc +
    lda #SND_CHOSEN
    jsr sound_play
+   rts
movement_a_press_selected:
++  ; move / attack
    lda tmp_range_attack
    bne ++
    bit tmp_is_flying
    bpl +
    jmp move_flying_creature
+   jmp move_walking_creature
++  jmp do_range_attack


move_walking_creature:
    ; TODO redundant? check computer.c ...
    lda acoord
    cmp scoord
    bne +
-   rts
+   ldx scoord
    jsr get_distance
    cmp #3 + 1
!if 1 {
    bcc move_creature
    rts
} else {
    bcs -
    jmp move_creature
}

move_flying_creature:
    ; TODO redundant? check computer.c ...
    lda acoord
    cmp scoord
    beq -
    bit tmp_engaged_flag
    bpl +
    lda #3
    sta tmp_movement_allowance
+
    lda acoord
    ldx scoord
    jsr get_distance
    cmp tmp_movement_allowance
    beq move_creature   ; allow dist == move
    bcs -
    ; dist < move
    ; FALL THROUGH

move_creature:
    sta distance_moved
    ; get the creature in the target square
    ldy acoord
    lda arena_above,y
    bne +
    ; no creature
--  jmp do_successful_move
+   ; is it dead?
    lda arena_dead,y
    and #DEAD_MASK_IS_DEAD
    bne --
    lda arena_above,y
    cmp #SPELL_MAGIC_WOOD
    bne move_creature_not_magic_wood
    ; if magic wood...
    lda selected_creature
    cmp #WIZARD_INDEX
    bcc +
    lda arena_below,y
    beq --
+   ; we are a wizard and the tree is occupied
    ; or we are a creature attacking a tree (can attack our own trees)
    jmp make_attack
move_creature_not_magic_wood:
    cmp #SPELL_SHADOW_WOOD
    beq move_creature_creature
    cmp #SPELL_MAGIC_FIRE
    bcs move_creature_not_creature
move_creature_creature:
    ; a creature or shadow wood
    lda arena_aflag,y
    and #AFLAG_MASK_OWNER
    cmp player_i
    beq +
    jmp make_attack
+   ; creature we are moving to belongs to current player
    ; check if we have a wizard selected for movement...
    lda selected_creature
    cmp #WIZARD_INDEX
    bcc +
    ; we are a wizard... check the target creature
    lda arena_above,y
    cmp #SPELL_BAT
    bcs +
    cmp #SPELL_HORSE
    bcs --  ; a mount, do move
-
+   rts
move_creature_not_creature:
    ; not an animal-type creature or shadow wood (i.e. could be a wizard or a citadel)
    cmp #SPELL_MAGIC_CASTLE
    bcc +
    cmp #SPELL_WALL
    bcc move_creature_magic_castle
+   ; not a magic castle/citadel, must be fire or wizard
    cmp #WIZARD_INDEX
    bcc -   ; must be fire - can't move here
    lda selected_creature
    cmp #WIZARD_INDEX
    bcs ++
    ; not a wizard selected, so check we aren't attacking our owner
    lda arena_above,y
    sec
    sbc #WIZARD_INDEX
    cmp player_i
    beq -
++  ; got to here?, we are a wizard attacking another wizard
    ; or a creature attacking an enemy wizard
    jmp make_attack
move_creature_magic_castle:
    ; a magic castle/citadel
    lda selected_creature
    cmp #WIZARD_INDEX
    bcc -   ; not a wizard selected, so can't go inside or attack
    lda arena_aflag,y
    and #AFLAG_MASK_OWNER
    cmp player_i
    bne -   ; not our castle so can't go in
    ; got to here? we are a wizard and the castle is ours, so move
    ; FALL THROUGH

do_successful_move:
    bit tmp_engaged_flag
    bpl +
    rts
+   lda selected_creature
    cmp #SPELL_SHADOW_WOOD
    bne +
--  jmp end_movement
+   lda #SND_WALK
    jsr sound_play
    ; flag that we need to remake the CPU movement table
    lda #0
    sta move_table_created
    ; tidy up the start square
    ldy scoord
    lda arena_aflag,y
    sta zptxt   ;start_a3
    lda arena_below,y
    sta zptxth  ;start_a4
    lda selected_creature
    cmp #WIZARD_INDEX
    bcc +
    cmp arena_above,y
    beq +
    ; the start creature is a wizard, but the visible thing is not
    ; i.e. wizard moving out of something
    lda #0
    sta zptxth  ;start_a4
    beq +++ ; bra
+   lda arena_bflag,y
    sta arena_above,y
    beq +++
    ; creature in arena_bflag??
    lda arena_dead,y
    ora #DEAD_MASK_IS_DEAD
    sta arena_dead,y
    lda #0
    sta arena_bflag,y   ; clear creature in arena_bflag (not sure about this)
+++ lda #0
    sta arena_below,y   ; nothing here
    ; sort out the target square
    ldy acoord
    lda arena_above,y
    beq +
    lda arena_dead,y
    and #DEAD_MASK_IS_DEAD
    beq ++
+   lda arena_above,y
    sta arena_bflag,y
    lda selected_creature
    sta arena_above,y
    lda zptxt   ;start_a3
    sta arena_aflag,y
    lda zptxth  ;start_a4
    sta arena_below,y
    lda arena_dead,y
    and #DEAD_MASK_OWNER    ; remember corpse owner while it is stepped on
    sta arena_dead,y
    jmp +++
++  ; something in target square and thing there is not dead
    ; must be a wiz moving to a mount or castle
    lda selected_creature
    sta arena_below,y
    ; end the wizard's move
    lda #0
    sta tmp_wizard_movement_allowance
    sta tmp_movement_allowance
+++ sty scoord
    lda attacker
    beq +
    jmp end_movement
+   lda arena_above,y
    cmp #SPELL_MAGIC_WOOD
    beq ++
    ; we have moved onto something that isn't a magic wood
    jsr check_engaged
    bcc +
    rts ; engaged
+   lda tmp_wizard_movement_allowance
    beq ++
    jmp end_movement
++  ; TODO was wait for letgo... does this add delay for CPU?
    bit tmp_is_flying
    bmi +
    lda tmp_movement_allowance
    sec
    sbc distance_moved
    beq +
    bcs ++
+   lda #0
    sta tmp_movement_allowance
    jmp end_movement
++  sta tmp_movement_allowance
    rts


; in:
; scoord = attacks
; acoord = defends
make_attack:
    lda #0
    sta attacking_val
    sta defending_val
    sta attacker_undead
    sta defender_undead
    sta attacker_modifier
    sta defender_modifier

    ldx scoord
    lda arena_above,x
    sta attacker
    jsr is_undead_x
    bcc +
    dec attacker_undead
+
    lda attacker
    cmp #WIZARD_INDEX
    bcc ++
    ; wizard attacking
    ; sec
    sbc #WIZARD_INDEX
    tay
    ldx wizard_offs,y
    ; remove shadow form
    lda player_data + PDATA_OFFS_FLAGS1,x
    and #!PDATA_MASK_FLAGS1_SHADOW
    sta player_data + PDATA_OFFS_FLAGS1,x
    and #PDATA_MASK_FLAGS1_SWORD | PDATA_MASK_FLAGS1_KNIFE | 1
    sta attacker_modifier
    beq +
    dec attacker_undead
+   lda player_data + PDATA_OFFS_RANGED,x
    beq ++
    ; possible bug here - if has magic bow, can melee attack undead
    dec attacker_undead
++

    ldx acoord
    lda arena_above,x
    sta defender
    jsr is_undead_x
    bcc +
    dec defender_undead
+
    lda defender
    cmp #WIZARD_INDEX
    bcc ++
    ; modify defence if defender has shield or armour
    ; sec
    sbc #WIZARD_INDEX
    tay
    ldx wizard_offs,y
    lda player_data + PDATA_OFFS_FLAGS1,x
    rol
    rol
    rol
    and #3
    tay
    beq +
    iny
+   ; if defender has shadow form, gets bonus 3 defence!
    lda player_data + PDATA_OFFS_FLAGS1,x
    and #PDATA_MASK_FLAGS1_SHADOW
    bne +
    iny
    iny
    iny
+   sty defender_modifier
++
    bit defender_undead
    bpl +++
    bit attacker_undead
    bmi +++
    bit flag_player_cpu
    bmi +
    ldy #STATUS3_UNDEAD
    jsr print_text_status3
-   jsr get_input
    beq -
+   rts
+++ ; get attacking val
    lda attacker
    cmp #WIZARD_INDEX
    bcc +
    ; sec
    sbc #WIZARD_INDEX
    tay
    ldx wizard_offs,y
    jmp ++
+   asl
    asl
    tax
++  ; x points to combat/def offset
    lda tmp_range_attack
    beq +
    inx ; x points to ranged_combat/range offset
+   lda attacker
    cmp #WIZARD_INDEX
    bcc +
    lda player_data + PDATA_OFFS_COMBAT_DEF,x
    jmp ++
+   lda creaturedata_packed,x
++  ; a has combat << 4 or rangedc << 4
    lsr
    lsr
    lsr
    lsr
    clc
    adc attacker_modifier   ; bug here - if the wizard has magic knife/sword, it improves his ranged combat!
    sta attacking_val
    jsr get_random_10
    clc
    adc attacking_val
    sta attacking_val
    ; get defending val
    lda defender
    cmp #WIZARD_INDEX
    bcc +
    ; sec
    sbc #WIZARD_INDEX
    tay
    ldx wizard_offs,y
    lda player_data + PDATA_OFFS_COMBAT_DEF,x
    jmp ++
+   asl
    asl
    tax
    lda creaturedata_packed,x
++  and #$f
    clc
    adc defender_modifier
    sta defending_val

    lda tmp_range_attack
    bne +
    lda #SND_ATTACK
    jsr sound_play
    ; do_attack_anim
    ldx #COLOR_RED
    ldy #GFX_ANIM_ATTACK
    jsr gfx_animation
+
    lda attacking_val
    cmp defending_val
    bcs +
    ; attack fails
    jmp end_movement
+   ; attack was a success
    ldx acoord
    lda arena_below,x
    bne +++
    ; nothing in arena_below
    lda defender
    cmp #WIZARD_INDEX
    bcc +
    ; was a wizard, do wizard death anim
    jsr kill_wizard
+   lda defender
    cmp #SPELL_VAMPIRE
    bcs +   ; do not leave corpse
    lda arena_aflag,x
    and #(AFLAG_MASK_ILLUSION | AFLAG_MASK_UNDEAD)
    bne +   ; do not leave corpse
    lda #$ff
    sta defender
+   jmp remove_creature
+++ ; there was something in the arena_below
    sta zptmp   ; arena_below[acoord]
    lda arena_above,x
    sta zptxt   ; arena_above[acoord]
    lda arena_aflag,x
    sta zptxth  ; arena_aflag[acoord]
    lda #0
    sta arena_below,x
    lda defender
    cmp #SPELL_GOOEY_BLOB
    bne +
    lda arena_bflag,x
    sta arena_aflag,x
    jmp ++
+   ; the famous "undead wizard" bug is caused by not updating the arena_aflag properly
    lda zptmp   ; arena_below[acoord]
    sec
    sbc #WIZARD_INDEX
    bcc ++
    sta arena_aflag,x
++  ; Another mini-bug here - if we kill a ridden mount, the wizard appears in
    ; the square (that's correct), but the mount leaves no corpse. Needs a fix?
    ; arena_bflag holds corpse info...
    lda zptmp   ; arena_below[acoord]
    sta arena_above,x
    lda zptxth  ; arena_aflag[acoord]
    and #(AFLAG_MASK_ILLUSION | AFLAG_MASK_UNDEAD)
    bne +   ; do not leave corpse
    lda zptxt   ; arena_above[acoord]
    cmp #SPELL_MANTICORE + 1
    bcc ++
+   lda #0
++  sta arena_bflag,x
    jmp end_movement

; remove the creature in acoord and move there
; in:
; defender = >=$80 if leaves corpse
remove_creature:
    ldx acoord
    lda #0
    sta tmp_engaged_flag
    sta tmp_movement_allowance
    sta tmp_wizard_movement_allowance
    bit defender
    bpl ++
    ; leaves corpse
    lda arena_aflag,x
    ora #DEAD_MASK_IS_DEAD
    sta arena_dead,x
    lda #12
    bne +++ ; bra
++  ; creature doesn't leave a corpse
    lda arena_bflag,x
    sta arena_above,x
    beq +
    lda arena_aflag,x
    ora #DEAD_MASK_IS_DEAD
    sta arena_dead,x
    lda #0
    sta arena_bflag,x
+   lda #6
+++ jsr delay
    lda tmp_range_attack
    beq +
    rts
+   lda #3
    sta distance_moved
    jmp do_successful_move


do_range_attack:
    ; have a target... attack!
    ldx acoord
    cpx scoord
    bne +
    ; can't attack self
-   rts
+   lda scoord
    jsr get_distance
    cmp tmp_range_attack
    bcs -   ; out of range, tmp_range_attack < distance
    ; in range, not attacking self...
    ldx #0
    lda scoord
    jsr los_blocked
    bcc -   ; no LOS
    lda #4
    jsr delay

    ; do the line animation
    lda #0
    sta Rangedtmp
    lda #SND_BEAM
    jsr sound_play
    ldx #COLOR_WHITE    ; TODO better color?
    lda selected_creature
    cmp #SPELL_DRAGON_GREEN
    bcc +
    cmp #SPELL_DRAGON_GOLDEN + 1
    bcs +
    ldx #COLOR_YELLOW   ; TODO better color?
    dec Rangedtmp
+   cmp #SPELL_MANTICORE
    bne +
    ldx #COLOR_RED  ; TODO better color?
+   lda scoord
    jsr los_blocked

    ; do the attack animation
    lda #SND_FIRE
    ldy #GFX_ANIM_BREATH
    ldx #COLOR_YELLOW   ; TODO better color?
    bit Rangedtmp
    bmi +
    lda #SND_SPELLSUCCESS
    ldy #GFX_ANIM_SPLAT
    ldx #COLOR_RED  ; TODO better color?
+   jsr sound_play
    jsr gfx_animation
    lda #4
    jsr delay

    ; do attack if a real target exists
    ldx acoord
    lda arena_dead,x
    and #DEAD_MASK_IS_DEAD
    bne ++
    lda arena_above,x
    beq ++
    cmp #SPELL_MAGIC_FIRE
    bcc +
    cmp #WIZARD_INDEX
    bcs +
    cmp #SPELL_MAGIC_WOOD
    beq +
    cmp #SPELL_SHADOW_WOOD
    bne ++
+   jsr make_attack
++
    lda #0
    sta selected_creature
    sta tmp_range_attack
    sta attacker
    rts


; out:
; C = 0
movement_phase_arena_view_cb:
    jsr print_clear_status3
    lda selected_creature
    bne +++
    ; no selected creature
    ; check if own creature and if it is ready for moving
    ldy acoord
    jsr movement_check_unmoved
    bcc +
    ldy #STATUS3_READY
    jmp print_text_status3
+
-   clc
    rts
+++ ; check/print oor if ((moving && !engaged) || ranged)
    ldy tmp_range_attack
    bne +++
    bit tmp_engaged_flag
    bmi -
    ; NOTE assumes tmp_movement_allowance > 0
    bit tmp_is_flying
    bmi +
    ; walking, step is 3
    ldy #3
    bne ++  ; bra
+   ldy tmp_movement_allowance
++  iny
+++ sty zptmp   ; current range (+ 1 if move)
    ; compare to selection range
    ldx scoord
    lda acoord
    jsr get_distance
    cmp zptmp   ; current range (+ 1 if move)
    bcc -
    ldy #STATUS3_OOR
    jmp print_text_status3


; Z = 0 if range attack remains
movement_cancel:
    ; movement_b
    lda tmp_range_attack
    bne +
    jsr end_movement
    lda tmp_range_attack
    rts
+   lda #0
    sta tmp_range_attack
    sta selected_creature
    rts


; end turn for creature at start_index
end_movement:
    ; whatever happens, the wizard has definitely moved
    lda #$ff
    sta has_wizard_moved
    ; and we definitely need to remake the movement table
    lda #0
    sta move_table_created
    sta attacker
    ; set has moved flag
    ldx scoord
    lda arena_aflag,x
    ora #AFLAG_MASK_MOVED
    sta arena_aflag,x
    ;jsr print_clear_statusline  ; TODO any real reason to clear here?
    lda arena_above,x
    cmp #SPELL_SHADOW_WOOD
    bne +
    lda #0
    sta attacker
--  lda #0  ; TODO redundant due to movement_clear_vars call in select_creature?
    sta tmp_range_attack
-   sta selected_creature
    rts
+   ; check range attack
    lda tmp_range_attack
    bne --
    lda arena_above,x
    beq -  ; TODO move earlier? NOTE does not clear tmp_range_attack on chaos-0.82
    cmp #WIZARD_INDEX
    bcc +
    ldx player_offs
    lda player_data + PDATA_OFFS_RANGED,x
    beq --
    bne +++ ; bra
+   ; creature ranged combat range * 2 + 1
    asl
    asl
    tax
    lda creaturedata_packed + 1,x
    beq --
    asl
    clc
    adc #1
+++ ; has ranged attack
    and #$f
    sta tmp_range_attack
    lda #SND_RANGE
    jsr sound_play
    bit flag_player_cpu
    bpl +
    jsr create_all_enemies_table
    lda #20
    jsr delay
 ;   rts
+;   jsr movement_print_status;_ranged
    rts


; in:
; selected_creature
; tmp_range_attack
; tmp_engaged_flag
; tmp_movement_allowance
; tmp_is_flying
movement_print_status:
    ; print creature / wizard name
    jsr print_clear_status
    jsr print_setup_dst_statusline
    lda selected_creature
    cmp #WIZARD_INDEX
    bcc +
    sec
    sbc #WIZARD_INDEX
    tay
    jsr print_wizard_name
    jmp ++
+   ldx player_i
    lda wizard_color,x
    sta color
    lda selected_creature
    jsr print_spellname_dst_ready
++  jsr print_setup_dst_next_line
    ; print move/fly/engaged/ranged
    lda tmp_range_attack
    beq ++
movement_print_ranged:
    jsr print_setup_dst_statusline1
    lda #COLOR_WHITE
    sta color
    lda #<text_range
    sta zptxt
    lda #>text_range
    sta zptxth
    jsr print_packed_dst_ready
    lda #COLOR_YELLOW
    sta color
    dex
    lda tmp_range_attack
    bne movement_print_status_number_2  ; bra
++  ; print move/fly/engaged
    bit tmp_engaged_flag
    bpl ++
    ; print engaged
    lda #<.text_engaged
    sta zptxt
    lda #>.text_engaged
    sta zptxth
    lda #COLOR_YELLOW
    sta color
    jsr print_packed_dst_ready
    rts
++  ; print move/fly
    lda #<.text_move
    ldx #>.text_move
    bit tmp_is_flying
    bpl +
    lda #<.text_fly
!if( >.text_fly) != (>.text_move) {
    ldx #>.text_fly
}
+   sta zptxt
    stx zptxth
    lda #COLOR_WHITE
    sta color
    jsr print_packed_dst_ready
    lda #COLOR_YELLOW
    sta color
    lda tmp_movement_allowance
    bit tmp_is_flying
    bmi +
    ; +1 from "movement points left"
    clc
    adc #1
+
movement_print_status_number_2:
    inx
    lsr
    jsr print_number
    rts


movement_find_next_creature:
    ldy acoord
--  iny
    cpy #AREA_SIZE
    bne +
    ldy #0
+   cpy acoord
    bne +
    rts
+   jsr movement_check_unmoved
    bcc --
    sty acoord
    rts


; in:
; y = coord
; out:
; C = 1 if unmoved movable creature
movement_check_unmoved:
    ; if something here and it is not dead and it is...
    ; (a creature, or a wizard, or a wizard inside a magic wood or castle) and
    ; it hasn't moved yet AND it is our creature or wizard...
    lda arena_dead,y
    and #DEAD_MASK_IS_DEAD
    beq +
--  clc
    rts
+   lda arena_aflag,y
    bmi --  ; has moved
    lda arena_above,y
    beq --
    cmp #SPELL_GOOEY_BLOB
    bcc ++
    cmp #WIZARD_INDEX
    bcs ++
    cmp #SPELL_SHADOW_WOOD
    beq ++
    cmp #SPELL_MAGIC_WOOD
    bcc --
    lda arena_below,y
    cmp #WIZARD_INDEX
    bcc --
++
    ; get the owner - low 3 bits of arena_aflag for creatures,
    ; (arena_above - WIZARD_INDEX) for wizards
    lda arena_above,y
    cmp #WIZARD_INDEX
    bcc +
    ; sec
    sbc #WIZARD_INDEX
    bpl +++  ; bra
+   ; a creature - but is it a wood/castle?
    cmp #SPELL_MAGIC_WOOD
    bcs +
    ; regular creature
    lda arena_aflag,y
    and #AFLAG_MASK_OWNER
    bpl +++ ; bra
+   ; a castle/wood - could be anyones, so base the owner on the wizard inside
    lda arena_below,y
    sec
    sbc #WIZARD_INDEX
+++ ; is it ours?
    cmp player_i
    bne --
    ; found unmoved creature
    sec
    rts


; in:
; acoord = target coord
; out:
; C = 1 if selected
select_creature:
    jsr movement_clear_vars
    ldy acoord
    sty scoord

    ; is there a creature here?
    lda arena_above,y
    sta selected_creature
    bne +
select_creature_nope:
    lda #0
    sta selected_creature
    clc
    rts
+   ; check if we have moved already
    lda arena_aflag,y
    bmi select_creature_nope
    lda selected_creature
    cmp #WIZARD_INDEX
    bcc +++
    ; sec
    sbc #WIZARD_INDEX
    cmp player_i
    bne select_creature_nope
    ; wizard is selected
    ldy #2  ; default
    ldx player_offs
    lda player_data + PDATA_OFFS_FLAGS1,x
    and #PDATA_MASK_FLAGS1_WINGS
    beq +
    dec tmp_is_flying
    ldy #$d
    bne ++  ; bra
+   lda player_data + PDATA_OFFS_FLAGS1,x
    and #PDATA_MASK_FLAGS1_SHADOW
    beq ++
    ldy #6
++  sty tmp_movement_allowance
    jmp select_creature_engaged

+++ ; a creature is selected
    ; set the movement value
    asl
    asl
    tay
    lda creaturedata_packed + 2,y
    lsr
    lsr
    lsr
    lsr
    asl
    sta tmp_movement_allowance
    lda selected_creature
    cmp #SPELL_PEGASUS
    bcc +
    cmp #SPELL_SPECTRE
    bcs +
    dec tmp_is_flying
    inc tmp_movement_allowance
+   ; check if there is something inside
    lda #0
    sta flag_anim   ; yes_pressed
    ldy acoord
    lda arena_below,y
    sta zptmp   ; inside_creature
    cmp #WIZARD_INDEX
    bcc +++
    ; sec
    sbc #WIZARD_INDEX
    cmp player_i
    bne select_creature_nope
    ; check if the covering creature is a wood or up
    lda selected_creature
    cmp #SPELL_MAGIC_WOOD
    bcs ++
    ; is not a wood, ask for dismount
    bit flag_player_cpu
    bpl +
    jsr cpu_dismount
    sta flag_anim   ; yes_pressed
    jmp select_creature_cond_dismount
+   ; ask player if dismount
    lda #COLOR_WHITE
    sta color
    jsr print_clear_status12
    ldx #<.text_dismountmenu
    ldy #>.text_dismountmenu
    jsr statusmenu
    cpy #1
    bne +++
    ; set the yes_pressed flag below
++  ; >= SPELL_MAGIC_WOOD
    ; a bit of a hack...
    dec flag_anim   ; yes_pressed

select_creature_cond_dismount:
    bit flag_anim   ; yes_pressed
    bpl +++
    ; dismount the wizard
    lda #3
    sta tmp_movement_allowance
    lda #1
    sta tmp_wizard_movement_allowance
    ; make sure we move the wizard
    lda zptmp   ; inside_creature
    sta selected_creature

+++ ; else inside creature < WIZARD_INDEX
    bit flag_anim   ; yes_pressed
    bmi select_creature_engaged
    lda selected_creature
    cmp #SPELL_SHADOW_WOOD
    beq +
    cmp #SPELL_GOOEY_BLOB
    bcc +
    jmp select_creature_nope    ; blob etc, so do nothing
+   ; get the owner
    ldy acoord
    lda arena_aflag,y
    and #AFLAG_MASK_OWNER
    cmp player_i
    beq +
    jmp select_creature_nope
+   lda arena_dead,y
    and #DEAD_MASK_IS_DEAD
    beq +
    jmp select_creature_nope
+
select_creature_engaged:
    jsr check_engaged
    bcc +
    rts
+   ; TODO WAS print movement range / flying
    sec
    rts


; out:
; C = 1 if engaged
check_engaged:
    lda #0
    sta tmp_engaged_flag
    ldx #7
-   lda scoord
    jsr apply_position_modifier
    jsr check_engaged_at
    bcs +
    dex
    bpl -
--  clc
    rts
+   ; is next to enemy
    ; if here, then get the arena_below value
    ; In ZX Chaos, mounted wizards can never be engaged to enemy
    ; change this so they can be, unless the wizard dismounts
    ldy scoord
    lda arena_below,y
    beq +
    lda selected_creature
    cmp #WIZARD_INDEX
    bcs --
+   ; compare maneouvre rating
    lda arena_above,y
    asl
    asl
    tay
    lda creaturedata_packed + 2,y
    and #$f
    sta .check_engaged_manvr
    jsr get_random_10
    clc
    adc #2
.check_engaged_manvr = * + 1
    cmp #$00
    bcc --
    ; set_engaged_to_enemy
    dec tmp_engaged_flag
    lda #1
    sta tmp_movement_allowance
    lda #0
    sta tmp_is_flying   ; if engaged, then we can't fly about
    ; TODO WAS print engaged
    sec
    rts

; in:
; a = coord
; out:
; C = 1 if engaged
check_engaged_at:
    cmp #$ff
    bne +
    clc
    rts
+   tay
    lda arena_dead,y
    and #DEAD_MASK_IS_DEAD
    beq +
-   clc
    rts
+   lda arena_above,y
    beq -
    cmp #SPELL_GOOEY_BLOB
    bcc +
    ; a blob or more
    sec
    sbc #WIZARD_INDEX
    bcc -
    cmp player_i
    beq -
    sec
    rts
+   ; an actual alive creature, get the owner
    lda arena_aflag,y
    and #AFLAG_MASK_OWNER
    cmp player_i
    beq -
    sec
    rts

} ; !zn Movement
